#!/usr/bin/python

import sys
import os
import time
import signal
import subprocess
import commands as cmd
import datetime
import time 
from _xmlplus.dom.NodeFilter import NodeFilter
from _xmlplus.dom import ext
from _xmlplus.dom import minidom
from datetime import datetime

def build_results(result, buffer):
    # Builds a results.xml by parsing xml logs in QTestLib xml format and 
    # combining their output into check xml format.


    
    # Then add those to the doc structure
    
    #testElement.setAttributeNS(checkns,"result","Success")

    #x = 0 
    #for line  in buffer.split("\n"):
    #    print "LINE " + str(x) + ": " + line
    #    x=x+1
    
    tmpdoc = minidom.parseString(buffer)

    e_testcase = tmpdoc.getElementsByTagName("TestCase")
    title =  e_testcase[0].attributes.item(0).nodeValue

    suiteElement = doc.createElementNS(checkns,"suite")
    titleElement = doc.createElementNS(checkns,"title")


    node = doc.createTextNode(title)
    titleElement.appendChild(node)

    rootElement.appendChild(suiteElement)
    suiteElement.appendChild(titleElement)


    suites = tmpdoc.getElementsByTagName("TestFunction")

    for elem in tmpdoc.getElementsByTagName("TestFunction"):
        testElement = doc.createElementNS(checkns,"test")
    
        pathElement = doc.createElementNS(checkns,"path")
        fnElement = doc.createElementNS(checkns,"fn")
        idElement = doc.createElementNS(checkns,"id")
        descriptionElement = doc.createElementNS(checkns,"description")
        messageElement = doc.createElementNS(checkns,"message")

        node = doc.createTextNode( elem.attributes.item(0).nodeValue )
        idElement.appendChild(node)

        node = doc.createTextNode(".")
        pathElement.appendChild(node)

        testElement.appendChild(pathElement)
        testElement.appendChild(fnElement)
        testElement.appendChild(idElement)
        testElement.appendChild(descriptionElement)
        testElement.appendChild(messageElement)
    
        suiteElement.appendChild(testElement)
        
        incident = elem.getElementsByTagName("Incident")
        if incident != None:
            incident_type = incident[0].getAttribute("type")
            incident_file = incident[0].getAttribute("file")
            incident_row  = incident[0].getAttribute("line")

            if (incident_type == "pass"):
                testElement.setAttributeNS(checkns,"result","success")
            elif (incident_type == "fail"):            
                testElement.setAttributeNS(checkns,"result","failure")
            else:
                testElement.setAttributeNS(checkns,"result","error")

            if incident_file != None and incident_file != "":
                node = doc.createTextNode(incident_file + ":" + str(incident_row))
            else:
                node = doc.createTextNode("n/a:0")
            
            fnElement.appendChild(node)


            description = incident[0].getElementsByTagName("DataTag")
            if description != None and description.length>0 and description[0].firstChild != None:
                node = doc.createTextNode(str(description[0].firstChild.data)+" ")
                descriptionElement.appendChild(node)

            description = incident[0].getElementsByTagName("Description")
            if description != None and description.length>0 and description[0].firstChild != None:
                node = doc.createTextNode(str(description[0].firstChild.data)+" ")
                descriptionElement.appendChild(node)

    #for binary in binariestorun:
    #    filename = "/tmp/" + binary + ".log"
    #    file = open(filename,"r")
    #    buffer = file.read()
    #    print binary
    
    return
    
def init_env(binariestorun):

    global doc
    global checkns 
    global rootElement 
    global datetimeElement 
    global durationElement
    global startTime

    doc = minidom.Document()
    doc = minidom.Document()
    checkns = "http://check.sourceforge.net/ns"
    rootElement = doc.createElementNS(checkns, "testsuites")
    datetimeElement = doc.createElementNS(checkns,"datetime")

    doc.appendChild(rootElement)
    rootElement.appendChild(datetimeElement)
    node = doc.createTextNode(datetime.today().strftime("%m.%d.%Y %H:%M:%S"))
    datetimeElement.appendChild(node)
    durationElement = doc.createElementNS(checkns,"duration")
    
    # If the binaries require different environments, which are not 
    # mutually compatible, init_env and deinit_env need to be called 
    # between running the binaries. Another option is to run the whole 
    # script several times with different arguments. 

    processes = []

    # Return a list of system processes that need to be closed after the
    # tests are ran.

    startTime = time.clock()
    return processes

def deinit_env(binariestorun, processes):

    endTime = time.clock()
    time.sleep(1)

    print "stop the daemonized processes", processes
    for process in processes:
        os.kill(process, signal.SIGTERM)
        os.waitpid(process, 0)

    rootElement.appendChild(durationElement)
    node = doc.createTextNode(str ( endTime - startTime) )
    durationElement.appendChild(node)

    file_object = open("/tmp/result.xml", "w")
    ext.PrettyPrint(doc, file_object)
    file_object.close()

def init_maps():

    # The test cases for test binaries. This might be automatically
    # generated?

    test_gprs = [ "test_connect" ]

    # Map the binary name keys to test case values ...
    casedict = { "test_gprs" : test_gprs }

    # ... and the other way around. There might be multiple binaries 
    # linked to one test case, so we need to have test binary lists for 
    # the test cases. Binarydict has test case names as keys, lists of 
    # corresponding binaries as values. 

    binarydict = {}

    for key in casedict.keys():
        for test in casedict[key]:
            if binarydict.has_key(test):
                binarylist = binarydict[test]
                binarylist.append(key)
            else:
                binarydict[test] = [key]

    return casedict, binarydict

def main():

    # 1. Initialize the mappings between the test cases and test 
    # binaries. This is needed if the test system wants to only specify 
    # test cases to be run. 
    
    casedict, binarydict = init_maps()

    # FIXME: temporary
    # The testmode variable changes the input handling behavior. If the 
    # variable is True, the input parameters are test case names. It 
    # the parameter is False, the input parameters are test binary 
    # names. 

    testmode = False
    # testmode = True

    binariestorun = []

    if testmode:
        # if input is test cases, see which binaries need to be run
        print "test cases: ", sys.argv[1:]
        for testcase in sys.argv[1:]:
            binarylist = binarydict[testcase]
            for binaryname in binarylist:
                if binariestorun.count(binaryname) == 0:
                    binariestorun.append(binaryname)
    
    else:
        # else if input is binary names, do nothing
        binariestorun = sys.argv[1:]


    # 2. Initialize the testing environment according to the binaries 
    # that are to be run. All system settings that differ from the 
    # basic boot-up state need to be defined here. 

    processes = init_env(binariestorun)

    # 3. Run the test binaries. It is assumed (for now) that they are 
    # in the same directory as the test script. 

    dirname = os.path.dirname(sys.argv[0])
    print "test binaries to run: ", binariestorun
    for binary in binariestorun:
        command = dirname + "/" + binary  + " -xml"
        # print command
        (status, output) =  cmd.getstatusoutput(command)
        cmdpipe = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=None, universal_newlines=True)
        status = cmdpipe.wait()
        output = cmdpipe.stdout.read()
        build_results(status,output)
        
    # 5. Clean up the testing environment.

    deinit_env(binariestorun, processes)

if __name__ == "__main__":
    sys.exit(main())




#!/usr/bin/python2.5
import sys,commands,re,os


def printxml(list,percentage):

    output = open('./coverage.log.xml','w')

    tests = len(list)
    fails = 0 
   
    
    print >>output, '<?xml version="1.0"?>'
    
    for row in list:
        (file,coveredp,coveredl,uncovered,excluded,reason) = row
        if coveredp < percentage:
            fails = fails + 1

    print >>output, '<testsuite name="CodeCoverage" tests="%d" failures="%d" errors="0" time="0.0">' % (tests,fails),

    for row in list:
        (file,coveredp,coveredl,uncovered,excluded,reason) = row
        print >>output 
        print >>output, '<testcase classname="line coverage" name="line coverage %s" time="0.0" ' % ( file ),
        if coveredp < percentage:
            print >>output, '>'
            if reason != None:
                msg = '0%% Coverage, Reason: %s' % (reason)
            else:
                msg  = 'Coverage: %s%% out of %s , (%d covered lines, %d uncovered, %d excluded)' %  ( str(coveredp), str(percentage),coveredl,uncovered,excluded )
            
            print >>output, ' <failure type="Standard" message="%s"/>' % ( msg )
            print >>output, '</testcase>',

        else:
            print >>output, '/>',

    print >>output,""
    print >>output, '</testsuite>'

    return fails 


def main(argv):
    if len(sys.argv) != 4:
        print "I need 3 arguments, " + str(len(sys.argv) - 1) + " given. "
        usage()

    # parse through arguments
    fileList = sys.argv[1].split(",")

    objectDirectory = sys.argv[2]


    results = []

    percentage = sys.argv[3]
    (t,y,exc) = (0,0,0)
    curdir = commands.getoutput("pwd")+"/"
    # run gcov for file1,file2,...
    results = []
    for singleFile in fileList:
        if singleFile != "":
            # singleFile = "../" + singleFile;
            singleFileWithoutPath = os.path.basename(singleFile)
            # print("cd .obj ; gcov --object-directory . " + singleFile );
            print ("gcov --object-directory .obj " + singleFile );
            commands.getoutput("gcov --object-directory . " + singleFile );
            #commands.getoutput("cd "+sd+";gcov --object-directory " +curdir+ objectDirectory +" "+ singleFile)
            try:
                #h= open(sd+i+".gcov","r") 
                h= open(singleFileWithoutPath + ".gcov","r");
            except:
                msg = "Unable to open file " + singleFileWithoutPath + ".gcov for reading"
                results.append( [singleFileWithoutPath,0,0,0,0,msg] )
            else:
                (e,r,o) = parse(h)
    
                name = singleFileWithoutPath
                coveredp = round(float(r)/(e+r)*100,2)
                coveredl = r
                uncovered =  e
                excluded = o
            
                results.append( [singleFileWithoutPath,coveredp,coveredl,uncovered,excluded,None] )
                print singleFileWithoutPath +  " covered "+str(round(float(r)/(e+r)*100,2))+" % ("+str(e)+" uncovered lines, "+str(r)+" covered lines, " + str(o) +" excluded lines, "+str(e+r+o)+" in total)" # print per-file coverage
        #(t,y,exc) = (t+e, r+y, exc+o) # sum up line counts

    ret = printxml(results,float(percentage))


    #print "Average coverage: " + str(round(float(y)/(t+y)*100,2))+" %" 
    #print "Total line coverage: "+str(t)+" uncovered lines, "+str(y)+" covered lines, " + str(exc) +" excluded lines, "+str(t+y+exc)+" in total)" # print per-file coverage
    # ret = (float(percentage)/100*(y+t) > y)
    if ret>0:
        print "error: code coverage below set limit"
    return 0

def usage():
    print "Usage: coverage.py file1,file2,... sourcedirect objectdirectory percentage"
    print
    print "Return values:"
    print " 0 check successful, coverage sufficient"
    print " 1 check successful, coverage insufficient"
    print " 2 check failed"
    sys.exit(2)

def parse (f):
    a = 0
    b = 0
    c = 0
    for i in f: # parse through the file
        s = i.split(":")[0];
        if s[-1]=="#":
            a += 1
        elif s[-1] != "-":
            b += 1
        else:
            c += 1    
    return (a,b,c); # return (uncovered lines, covered lines count)

sys.exit(main(sys.argv))
